<?php
namespace Drupal\pagedesigner_block\Plugin\pagedesigner\Handler;

use Drupal\block\Entity\Block as BlockEntity;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Render\Markup;
use Drupal\pagedesigner\Definition\PatternDefinition;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\HandlerPluginBase;

/**
 * @PagedesignerHandler(
 *   id = "block",
 *   name = @Translation("Block handler"),
 *   types = {
 *      "block"
 *   },
 * )
 */
class Block extends HandlerPluginBase
{
    protected $definition = [
        "id" => 'block:',
        "pagedesigner" => 1,
        "icon" => "fab fa-drupal",
        "type" => "block",
        "category" => 'Drupal blocks',
        "label" => '',
        "description" => '',
    ];

    public function collectAttachments(&$attachments)
    {
        $attachments['library'][] = 'pagedesigner_block/pagedesigner';
    }

    /**
     * {@inheritDoc}
     */
    public function collectPatterns(&$patterns)
    {
        $blockIds = \Drupal::entityQuery('block')->condition('region', 'pagedesigner')->condition('theme', \Drupal::service('theme.manager')->getActiveTheme()->getName())->execute();
        foreach ($blockIds as $blockId) {
            $definition = $this->definition;
            $id = $definition['id'] . $blockId;
            $cid = 'pagedesigner_definition_block_' . $blockId;
            if ($item = \Drupal::cache()->get($cid)) {
                $patterns[$id] = new PatternDefinition($item->data);
            } else {
                $block = BlockEntity::load($blockId);
                if ($block != null) {
                    $definition['block'] = $blockId;
                    $build = \Drupal::entityTypeManager()
                        ->getViewBuilder('block')
                        ->view($block);
                    $markup = \Drupal::service('renderer')->renderPlain($build);
                    $markup = '<div data-gjs-type="block">' . $markup . '<div>';
                    $definition['markup'] = $markup;
                    $definition['label'] = (string) $block->label();
                    $patterns[$id] = new PatternDefinition($definition);
                    \Drupal::cache()->set($cid, $definition, Cache::PERMANENT, $block->getCacheTags());
                }
            }
        }
    }

    public function get(Element $entity)
    {
        // TODO: Find a minimal representation of a block for search
        return '';
    }

    public function render(Element $entity)
    {
        if ($entity->field_block->entity == null) {
            return [];
        }
        return \Drupal::entityManager()
            ->getViewBuilder('block')
            ->view($entity->field_block->entity);
    }

    /**
     * {@inheritdoc}
     */
    public function renderForEdit(Element $entity)
    {
        $build = $this->render($entity);
        $markup = \Drupal::service('renderer')->render($build);

        return [
            '#type' => 'inline_template',
            '#template' => '<div data-gjs-type="block" data-entity-id="{{id}}" id="{{html_id}}">{{markup}}</div>',
            '#context' => ['markup' => ['#markup' => Markup::create($markup)], 'id' => $entity->id(), 'html_id' => 'pd-cp-' . $entity->id()],
        ];
    }

    /**
     * {@inheritDoc}
     */
    public function generate($patternDefinition, $data)
    {
        $build = [];
        $build['type'] = $type = $patternDefinition->getAdditional()['type'];

        $blockId = $patternDefinition->getAdditional()['block'];
        $block = BlockEntity::load($blockId);
        if ($block == null) {
            throw new BadRequestHttpException('The given block does not exist.');
        }
        $element = Element::create(['type' => 'block', 'name' => 'block']);
        $element->field_block->entity = $block;
        $element->parent->target_id = $data['parent'];
        $element->container->target_id = $data['container'];
        $element->entity->target_id = $data['entity'];
        $element->saveEdit();

        // Collect return data
        $build['id'] = $element->id();

        // Adding entity to parent cell if provided
        $cell = Element::load($element->parent->target_id);
        $language = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
        $cell = $cell->getTranslation($language);
        if ($cell != null) {
            $cell->children->appendItem($element);
            $cell->saveEdit();
        }

        return $build;

    }
}
