<?php

namespace Drupal\pagedesigner;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Language\LanguageManager;
use Drupal\Core\Session\AccountInterface;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface;
use Drupal\pagedesigner\Service\HandlerPluginManager;

class PagedesignerService
{
    /**
     * The output of the last operation.
     *
     * @var mixed
     */
    protected $_output = null;

    /**
     * The language manager
     *
     * @var \Drupal\Core\Language\LanguageManager
     */
    protected $languageManager = null;

    /**
     * The handler manager
     *
     * @var \Drupal\pagedesigner\Service\HandlerPluginManager
     */
    protected $handlerManager = null;

    /**
     * Create a new instance.
     *
     * @param AccountInterface $user The user for which to process. If empty, the current user is used.
     * @param string $langCode The language to process in. If empty, the current language is used.
     */
    public function __construct(LanguageManager $language_manager, HandlerPluginManager $handler_manager)
    {
        $this->languageManager = $language_manager;
        $this->handlerManager = $handler_manager;
    }

    /**
     * Returns the output of the last operation.
     *
     * @return mixed
     */
    public function getOutput()
    {
        return $this->_output;
    }

    public function getPagedesignerFields(ContentEntityBase $entity)
    {
        $pagedesignerFields = [];
        $fields = $entity->getFields(true);
        foreach ($fields as $field) {
            if ($field->getFieldDefinition()->getType() == 'pagedesigner_item') {
                $pagedesignerFields[$field->getName()] = $field;
            }
        }
        return $pagedesignerFields;
    }

    /**
     * Adds the container value to the given entity.
     * If no container exists, a container will be created.
     *
     * @param ContentEntityBase $entity
     * @return void
     */
    public function addContainer(ContentEntityBase &$entity)
    {
        $entityManager = \Drupal::service('entity_field.manager');
        $fields = $entityManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
        foreach ($fields as $field) {
            if ($field->getType() == 'pagedesigner_item') {
                if ($entity->get($field->getName())->isEmpty()) {
                    $container = $this->getContainer($entity, $field->getName());
                    $entity->get($field->getName())->entity = $container;
                } else if (!$entity->isDefaultTranslation()) {
                    $container = $entity->get($field->getName())->entity;
                    $language = $entity->language()->getId();
                    if (!$container->hasTranslation($language)) {
                        $container->addTranslation($language);
                        $container->save();
                    }
                }
            }
        }
    }

    /**
     * Adds the container value to the given entity.
     * If no container exists, a container will be created.
     *
     * @param ContentEntityBase $entity
     * @return void
     */
    public function saveContainer(ContentEntityBase &$entity)
    {
        $entityManager = \Drupal::service('entity_field.manager');
        $fields = $entityManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
        foreach ($fields as $field) {
            if ($field->getType() == 'pagedesigner_item') {
                if (!$entity->get($field->getName())->isEmpty()) {
                    $container = $entity->get($field->getName())->entity;
                    $container->entity = $entity;
                    $container->save();
                }
            }
        }

    }

    /**
     * Loads or creates the container for the given entity in the language of the entity.
     *
     * @param ContentEntityBase $entity The entity being rendered.
     * @return Element The container.
     */
    public function getContainer(ContentEntityBase $entity, $fieldname = 'field_pagedesigner_content')
    {
        $container = $entity->get($fieldname)->entity;
        $language = $entity->language()->getId();
        if ($container == null) {
            $container = $this->createContainer($language);
        } else {
            $container = $this->getLanguage($container, $language);
        }
        return $container;
    }

    /**
     * Create a new container.
     *
     * @return void
     */
    public function createContainer($language)
    {
        $container = Element::create(['type' => 'container', 'name' => 'container']);
        $container->langcode->value = $language;
        $container->save();
        return $container;
    }

    /**
     * Undocumented function
     *
     * @param Element $container
     * @param string $language
     * @return void
     */
    public function getLanguage(Element $container, $language)
    {
        if (!$container->hasTranslation($language)) {
            $container->addTranslation($language);
            $container->save();
        }
        return $container->getTranslation($language);
    }

    /**
     * Get the handler.
     *
     * @param ContentEntityBase $entity
     * @return Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface
     */
    protected function getHandler(ContentEntityBase $entity)
    {
        return $this->handlerManager->createInstance($entity->bundle());
    }
}
