<?php
namespace Drupal\pagedesigner\Service;

use Drupal\Component\Plugin\FallbackPluginManagerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Session\AccountInterface;
use Drupal\node\Entity\Node;

/**
 * HandlerPluginManager manager.
 *
 * This class provides static functions for retrieving node renderers.
 * For most use cases one needs to get the appropiate renderer using one of the static methods and process the node using the node renderer.
 *
 * @see \Drupal\Core\Archiver\Annotation\Archiver
 * @see \Drupal\Core\Archiver\ArchiverInterface
 * @see plugin_api
 */
class HandlerPluginManager extends DefaultPluginManager implements FallbackPluginManagerInterface
{
    /**
     * The drupal database connection object.
     *
     * @var Connection
     */
    protected $connection = null;

    /**
     * Temporary cache
     *
     * The nodes loaded from database during processing.
     *
     * @var Node[]
     */
    protected $revisions = [];

    /**
     *
     */
    public $currentNode = null;

    /**
     * Constructs a RendererPluginManager object.
     *
     * @param \Traversable $namespaces
     *   An object that implements \Traversable which contains the root paths
     *   keyed by the corresponding namespace to look for plugin implementations.
     * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
     *   Cache backend instance to use.
     * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
     *   The module handler to invoke the alter hook with.
     */
    public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler)
    {
        parent::__construct(
            'Plugin/pagedesigner/Handler',
            $namespaces,
            $module_handler,
            'Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface',
            'Drupal\pagedesigner\Annotation\PagedesignerHandler'
        );
        $this->alterInfo('pagedesigner_handler_info');
        $this->setCacheBackend($cache_backend, 'pagedesigner_handler_info_plugins');
    }

    /**
     * Overrides PluginManagerBase::getInstance().
     *
     * @param array $options
     *   An array with the following key/value pairs:
     *   - id: The id of the plugin.
     *   - type: The type of the pattern field.
     *
     * @return \Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface[]
     *   A list of Handler objects.
     */
    public function getInstance(array $options)
    {
        $handlers = [];
        $type = '*';
        if (!empty($options['entity'])) {
            $type = $options['entity']->bundle();
        } else {
            $type = $options['type'];
        }
        $configuration = []; //'type' => $type, 'entity' => $options['entity']];
        foreach ($this->getDefinitions() as $plugin_id => $definition) {
            if (in_array($type, $definition['types'])) {
                $handlers[] = $this
                    ->createInstance($plugin_id, $configuration);
            }
        }
        if (empty($handlers)) {
            $handlers[] = $this
                ->createInstance('default', $configuration);
        }
        return $handlers;
    }

    /**
     * @return \Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface[]
     *   A list of Handler objects.
     */
    public function getHandlers()
    {
        $handlers = [];
        $configuration = [];
        foreach ($this->getDefinitions() as $plugin_id => $definition) {
            $handlers[] = $this
                ->createInstance($plugin_id, $configuration);
        }
        return $handlers;
    }

    /**
     * {@inheritDoc}
     */
    public function getFallbackPluginId($plugin_id, array $configuration = array())
    {
        return 'standard';
    }
}
