<?php

namespace Drupal\pagedesigner\Service;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Render\Markup;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\PagedesignerService;
use Drupal\pagedesigner\Service\Render\Styles;
use Drupal\user\Entity\User;

class Renderer extends PagedesignerService
{
    /**
     * Style object to collect the styles during rendering.
     *
     * @var Drupal\pagedesigner\Service\Render\Styles|null
     */
    protected static $styles = null;

    /**
     * Add a style definition.
     *
     * @param string $key The size for which to add the style definition (mn, sm, xs).
     * @param string $style The style definition to add.
     * @param int $id The id of the element.
     * @return void
     */
    public static function addStyle($key, $style, $id)
    {
        if (self::$styles == null) {
            self::$styles = new Styles();
        }
        self::$styles->addStyle($key, $style, $id);
    }

    /**
     * Render a container for a certain entity.
     *
     * @param Element $container
     * @param ContentEntityBase $entity
     * @return void
     */
    public function render(Element $container, ContentEntityBase $entity = null)
    {
        if ($container == null) {
            return $this;
        }
        $this->_output = $this->getHandler($container)->render($container);
        if (empty($this->_output['#attached'])) {
            $this->_output['#attached'] = [];
        }
        $this->_output['#attached']['library'][] = 'pagedesigner/icon';
        $this->_output['#cache']['max-age'] = 0;
        $this->addStyles();
        return $this;
    }

    /**
     * Publicly render a container for a certain entity.
     *
     * @param Element $container
     * @param ContentEntityBase $entity
     * @return void
     */
    public function renderForPublic(Element $container, ContentEntityBase $entity = null)
    {
        if ($container == null) {
            return $this;
        }
        $this->_output['#cache'] =
            ['keys' => ['entity_view', 'node', $entity->id()],
            'contexts' => ['languages'],
            'tags' => $entity->getCacheTags(),
            'max-age' => Cache::PERMANENT,
        ];
        $this->_output = $this->getHandler($container)->renderForPublic($container);
        $this->addStyles();
        return $this;
    }

    /**
     * Editebly render a container for a certain entity.
     *
     * @param Element $container
     * @param ContentEntityBase $entity
     * @return void
     */
    public function renderForEdit(Element $container, ContentEntityBase $entity = null)
    {
        if ($container == null) {
            return $this;
        }

        if ($entity != null) {
            $locker = \Drupal::service('pagedesigner.locker');
            $locker->setEntity($entity);

            if (isset($_GET['otherTab']) && $_GET['otherTab'] == 1) {
                $username = 'unknown';
                $otherUser = User::load($locker->getMetaData()->owner);
                if ($otherUser != null) {
                    $username = $otherUser->getUsername();
                }
                drupal_set_message(t('This page is currently being edited by you in another tab. Continue editing in that tab or close the other tab and reload here.', []), 'warning');
                return $this->render($container, $entity);
            }
            $lock = $locker->acquire();
            $lockIdentifier = $locker->getIdentifier();
            if (!$lock && !$locker->hasLock()) {
                $username = 'unknown';
                $otherUser = User::load($locker->getMetaData()->owner);
                if ($otherUser != null) {
                    $username = $otherUser->getUsername();
                }
                drupal_set_message(t('This page is currently being edited by user %user. You may access it again after the user is done editing.', array('%user' => $username)), 'warning');
                return $this->render($container, $entity);
            }
        }

        $this->_output = $this->getHandler($container)->renderForEdit($container);
        $this->_output['#cache']['max-age'] = 0;
        $this->addStyles();
        $this->addEditorAttachments();
        $this->addPagedesigner();
        $this->addPatternAttachments();
        return $this;
    }

    /**
     * Return the markup generated by the last render.
     *
     * @return void
     */
    public function getMarkup()
    {
        return Markup::create(\Drupal::service('renderer')->render($this->_output));
    }

    /**
     * Return the styles generated by the last render.
     *
     * @return void
     */
    public function getStyles()
    {
        $rendered = '';
        if (self::$styles != null) {
            foreach (['large' => '', 'medium' => '@media (max-width: 992px)', 'small' => '@media (max-width: 768px)'] as $size => $query) {
                if (empty($query)) {
                    $rendered .= self::$styles->getStyles($size) . "\n";
                } else {
                    $rendered .= $query . '{' . self::$styles->getStyles($size) . '}' . "\n";
                }
            }
        }
        return $rendered;
    }

    /**
     * Add the collected styles of the elements to the output.
     *
     * @return void
     */
    protected function addStyles()
    {
        if (self::$styles != null) {
            $rendered = '';
            foreach (['large' => '', 'medium' => '@media (max-width: 992px)', 'small' => '@media (max-width: 768px)'] as $size => $query) {
                if (empty($query)) {
                    $rendered .= self::$styles->getStyles($size) . "\n";
                } else {
                    $rendered .= $query . '{' . self::$styles->getStyles($size) . '}' . "\n";
                }
            }
            $this->_output['#attached']['html_head']['pd_styles'] = [
                [
                    '#tag' => 'style',
                    '#value' => $rendered,
                    '#attributes' => [
                        'id' => 'pd_styles',
                    ],
                ],
                'pagedesigner_dynamic_css',
            ];
        }
    }

    /**
     * Add attachments for the WYSIWYG editor to the output.
     *
     * @return void
     */
    protected function addEditorAttachments()
    {
        if (empty($this->_output['#attached'])) {
            $this->_output['#attached'] = [];
        }
        $formats = ['pagedesigner'];
        $this->_output['#attached'] = array_merge(
            $this->_output['#attached'],
            \Drupal::service('plugin.manager.editor')->getAttachments($formats)
        );
    }

    /**
     * Adds the basic pagedesigner edit and pattern libraries needed to edit to the pagedesigner to the output.
     *
     * @return void
     */
    protected function addPagedesigner()
    {
        if (empty($this->_output['#attached'])) {
            $this->_output['#attached'] = [];
        }
        $this->_output['#attached']['library'][] = 'pagedesigner/pagedesigner';
        $handlers = \Drupal::service('plugin.manager.pagedesigner_handler')->getHandlers();
        foreach ($handlers as $handler) {
            $handler->collectAttachments($this->_output['#attached']);
        }
    }

    /**
     * Adds the attachments of all pattens to the output.
     *
     * @return void
     */
    protected function addPatternAttachments()
    {
        if (empty($this->_output['#attached'])) {
            $this->_output['#attached'] = [];
        }

        $patternDefinitions = \Drupal::service('plugin.manager.ui_patterns')->getDefinitions();
        $handlers = \Drupal::service('plugin.manager.pagedesigner_handler')->getHandlers();
        foreach ($handlers as $handler) {
            $handler->collectPatterns($patternDefinitions);
        }

        foreach ($patternDefinitions as $definition) {
            if ($definition->getProvider() !== \Drupal::service('theme.manager')->getActiveTheme()->getName()) {
                continue;
            }
            foreach ($definition->getLibrariesNames() as $library) {
                $this->_output['#attached']['library'][] = $library;
            }
        }
    }
}
